/**
 *
 * Project: Eight Track, File: Mp3UploadHandler.java
 *
 * Copyright 2019-2020 Locusworks LLC.
 * All rights reserved. Federal copyright law prohibits unauthorized reproduction by
 * any means and imposes fines up to $25,000 for violation.  No part of this material
 * may be reproduced, transmitted, transcribed, stored in a retrieval system, copied,
 * modified, duplicated, adapted or translated into another program language in any
 * form or by any means, electronic, mechanical, photocopying, recording, or
 * otherwise, without the prior written permission from Locusworks. Locusworks
 * affirms that Eight-Track(R) software and data is subject to United States
 * Government Purpose Rights.  Contact Locusworks, 1313 Lawnview Drive
 * Forney TX 75126, (802) 488-0438, for commercial licensing opportunities.
 *
 * IN NO EVENT SHALL LOCUSWORKS BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL,
 * INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LOCUSWORKS HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  NO RESPONSIBILITY IS ASSUMED BY
 * LOCUSWORKS FOR ITS USE, OR FOR ANY INFRINGEMENTS OF PATENTS OR OTHER RIGHTS OF
 * THIRD PARTIES RESULTING FROM ITS USE.  LOCUSWORKS SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE AND
 * ACCOMPANYING DOCUMENTATION, IF ANY, PROVIDED HEREUNDER IS PROVIDED "AS
 * IS". LOCUSWORKS HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 * ENHANCEMENTS, OR MODIFICATIONS.
 */
package net.locusworks.discord.eighttrack.audio;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.UUID;
import org.jaudiotagger.audio.AudioFile;
import org.jaudiotagger.audio.AudioFileIO;
import org.jaudiotagger.tag.Tag;
import org.springframework.stereotype.Component;

@Component
public class Mp3UploadHandler {

  public Mp3UploadResults parse(Path file) throws IOException {
    InputStream is = Files.newInputStream(file);
    return parse(is, file.getFileName().toString());
  }

  public Mp3UploadResults parse(InputStream is, String fileName) throws IOException {
    byte[] data;
    Path tmp = null;
    try(InputStream in = is; ByteArrayOutputStream baos = new ByteArrayOutputStream()) {
      int read = 0;
      byte[] buffer = new byte[1024 * 1024];
      while((read = in.read(buffer)) > 0) {
        baos.write(buffer, 0, read);
      }
      data = baos.toByteArray();
      
      String id = UUID.nameUUIDFromBytes(data).toString();
      
      tmp = Paths.get(String.format("%s_%s", id, fileName));
      
      Files.write(tmp, data);
      
      AudioFile f = AudioFileIO.read(tmp.toFile());
      Tag tag = f.getTag();
      
      return new Mp3UploadResults(tag, data);
    } catch (Exception ex) {
      return new Mp3UploadResults(ex);
    } finally {
      if (tmp != null) {
        Files.deleteIfExists(tmp);
      }
    }
  }
}
