/**
 *
 * Project: Eight Track, File: AdminPlaylistListHandler.java
 *
 * Copyright 2019-2020 Locusworks LLC.
 * All rights reserved. Federal copyright law prohibits unauthorized reproduction by
 * any means and imposes fines up to $25,000 for violation.  No part of this material
 * may be reproduced, transmitted, transcribed, stored in a retrieval system, copied,
 * modified, duplicated, adapted or translated into another program language in any
 * form or by any means, electronic, mechanical, photocopying, recording, or
 * otherwise, without the prior written permission from Locusworks. Locusworks
 * affirms that Eight-Track(R) software and data is subject to United States
 * Government Purpose Rights.  Contact Locusworks, 1313 Lawnview Drive
 * Forney TX 75126, (802) 488-0438, for commercial licensing opportunities.
 *
 * IN NO EVENT SHALL LOCUSWORKS BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL,
 * INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LOCUSWORKS HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  NO RESPONSIBILITY IS ASSUMED BY
 * LOCUSWORKS FOR ITS USE, OR FOR ANY INFRINGEMENTS OF PATENTS OR OTHER RIGHTS OF
 * THIRD PARTIES RESULTING FROM ITS USE.  LOCUSWORKS SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE AND
 * ACCOMPANYING DOCUMENTATION, IF ANY, PROVIDED HEREUNDER IS PROVIDED "AS
 * IS". LOCUSWORKS HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 * ENHANCEMENTS, OR MODIFICATIONS.
 */
package net.locusworks.discord.eighttrack.events.admin.handlers;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.events.message.guild.GuildMessageReceivedEvent;
import net.locusworks.discord.eighttrack.database.entities.GuildPlaylist;
import net.locusworks.discord.eighttrack.events.admin.AbstractAdminEventHandler;
import net.locusworks.discord.eighttrack.services.RepositoryService;

@Component
public class AdminPlaylistListHandler extends AbstractAdminEventHandler {
  @Autowired
  private RepositoryService guildRepoService;

  public AdminPlaylistListHandler() {
    super("list");
  }
  
  @Override
  public void executeEvent(GuildMessageReceivedEvent event, List<String> commands) {
    if (!isAdmin(event)) return;
    
    List<GuildPlaylist> userPlaylist = guildRepoService.getGuildPlaylistRepo().findByGuildFetchSongs(event.getGuild().getIdLong());
    if (userPlaylist == null || userPlaylist.isEmpty()) {
      event.getChannel().sendMessage(event.getAuthor().getAsMention() + ", there are no playlists for this guild").queue();
      return;
    }
    int longestPlaylist = 10;
    int longestUserName = 10;
    Map<Long, String> userMap = new HashMap<>();
    for(GuildPlaylist gpl : userPlaylist) {
      if (gpl.getPlaylist().length() > longestPlaylist) longestPlaylist = gpl.getPlaylist().length();

      Member m = event.getGuild().getMemberById(gpl.getUserId());
      if (m == null) continue;

      if (m.getUser().getAsTag().length() > longestUserName) longestUserName = m.getUser().getAsTag().length();
      userMap.put(gpl.getUserId(), m.getUser().getAsTag());
    }

    String fmt = "%6s | %-" + longestPlaylist +"s | %-" + longestUserName + "s | %s%n"; 

    StringBuilder sb = new StringBuilder("```");
    sb.append(String.format(fmt, "", "Playlist", "User", "Song Count"));
    sb.append(StringUtils.repeat("-", 24 + longestPlaylist + longestUserName)).append("\n");

    int count = 0;
    for(GuildPlaylist gpl : userPlaylist) {
      sb.append(String.format(fmt, ++count, gpl.getPlaylist(), userMap.get(gpl.getUserId()), gpl.getGuildPlaylistSongList().size()));
    }
    sb.append("```");
    event.getChannel().sendMessage("**Playlists for " + event.getGuild().getName() + "**").queue((succcess)-> 
    event.getChannel().sendMessage(sb.toString()).queue());
  }
  
  @Override
  public String help() {
    return "list" ;
  }

}
