/**
 *
 * Project: Eight Track, File: AdminPlaylistHandler.java
 *
 * Copyright 2019-2020 Locusworks LLC.
 * All rights reserved. Federal copyright law prohibits unauthorized reproduction by
 * any means and imposes fines up to $25,000 for violation.  No part of this material
 * may be reproduced, transmitted, transcribed, stored in a retrieval system, copied,
 * modified, duplicated, adapted or translated into another program language in any
 * form or by any means, electronic, mechanical, photocopying, recording, or
 * otherwise, without the prior written permission from Locusworks. Locusworks
 * affirms that Eight-Track(R) software and data is subject to United States
 * Government Purpose Rights.  Contact Locusworks, 1313 Lawnview Drive
 * Forney TX 75126, (802) 488-0438, for commercial licensing opportunities.
 *
 * IN NO EVENT SHALL LOCUSWORKS BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL,
 * INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LOCUSWORKS HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  NO RESPONSIBILITY IS ASSUMED BY
 * LOCUSWORKS FOR ITS USE, OR FOR ANY INFRINGEMENTS OF PATENTS OR OTHER RIGHTS OF
 * THIRD PARTIES RESULTING FROM ITS USE.  LOCUSWORKS SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE AND
 * ACCOMPANYING DOCUMENTATION, IF ANY, PROVIDED HEREUNDER IS PROVIDED "AS
 * IS". LOCUSWORKS HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 * ENHANCEMENTS, OR MODIFICATIONS.
 */
package net.locusworks.discord.eighttrack.events.main.handlers;

import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import net.dv8tion.jda.api.events.message.guild.GuildMessageReceivedEvent;
import net.locusworks.discord.eighttrack.events.admin.AbstractAdminEventHandler;
import net.locusworks.discord.eighttrack.events.main.AbstractMainEventHandler;
import net.locusworks.discord.eighttrack.services.RepositoryService;
import net.locusworks.logger.ApplicationLogger;
import net.locusworks.logger.ApplicationLoggerFactory;

@Component
public class AdminPlaylistHandler extends AbstractMainEventHandler {

  private ApplicationLogger logger;
  
  @Autowired
  private RepositoryService guildRepoService;

  private Map<String, AbstractAdminEventHandler> handlers;
  
  public AdminPlaylistHandler() {
    super("+playlist");
    logger = ApplicationLoggerFactory.getLogger(AdminPlaylistHandler.class);
  }
  
  @Autowired
  private void init(List<AbstractAdminEventHandler> eventHandlers) {
    handlers = new TreeMap<>();
    for(AbstractAdminEventHandler apeh : eventHandlers) {
      handlers.put(apeh.getCommand(), apeh);
    }
  }

  @Override
  public void executeEvent(GuildMessageReceivedEvent event, List<String> commands) {
    if (!isAdmin(event)) return;
    
    if (commands == null || commands.isEmpty()) {
      event.getChannel().sendMessage("Missing command for +playlist. Valid commands are: list, delete").queue();
      return;
    }
    try {
      String command = commands.remove(0);
      
      AbstractAdminEventHandler apeh = handlers.get(command);
      if (apeh == null) {
        sendMessage(event, help());
        return;
      }
      
      apeh.executeEvent(event, commands);
      
    } catch (Exception ex) {
      logger.error("Unable to display playlist: " + ex.getMessage(), ex);
      guildRepoService.logEntry(event.getGuild().getIdLong(), ex, "Unable to display playlist: %s", ex.getMessage());
      sendMessage(event, "Sorry I am unable to display the playlist: " + ex.getMessage());
    }
  }
  
  @Override
  public String help() {
    StringBuilder sb = new StringBuilder();
    for(Entry<String, AbstractAdminEventHandler> entry : handlers.entrySet()) {
      sb.append(entry.getValue().help()).append("\n");
    }
    
    return "+playlist | " + sb.toString();
  }

}
